package com.winhc.bigdata.spark.utils

import java.text.SimpleDateFormat

import org.apache.commons.lang3.StringUtils

/**
 * @Author: XuJiakai
 * @Date: 2020/8/10 11:19
 * @Description:
 */
object DateUtils {

  def splitDate(date: String): String = {
    try {
      if (StringUtils.isEmpty(date)) {
        null
      } else {
        date.split(" ")(0)
      }
    } catch {
      case ex: Exception => null
    }
  }

  private def addZero(str: String): String = {
    if (str.length == 2) {
      str
    } else {
      "0" + str
    }
  }

  def formatterDate(date: String): String = {
    try {
      var d = date.replaceAll("[年月日号/]", "-")
      if (d.contains(" ")) {
        d = d.split(" ")(0)
      }
      val ymd = d.split("-")
      s"${ymd(0)}-${addZero(ymd(1))}-${addZero(ymd(2))}"
    } catch {
      case ex: Exception => {
        println(date)
      }
        date
    }
  }

  def toMillisTimestamp(date: String, pattern: String = "yyyy-MM-dd HH:mm:ss"): String = {
    if (StringUtils.isEmpty(date)) {
      return null
    }

    var p = "yyyy-MM-dd HH:mm:ss"
    if (date.length == 10) {
      p = "yyyy-MM-dd"
    }
    val fm = new SimpleDateFormat(p)
    fm.parse(date).getTime + ""
  }

  def toUnixTimestamp(date: String, pattern: String = "yyyy-MM-dd HH:mm:ss"): Long = {
    var p = "yyyy-MM-dd HH:mm:ss"
    if (date.length == 10) {
      p = "yyyy-MM-dd"
    }
    val fm = new SimpleDateFormat(p)
    fm.parse(date).getTime / 1000
  }

  /**
   * 获取第一个不为空的字符串
   *
   * @param date
   * @return
   */
  def getNotNullStr(date: String*): String = {
    if (date == null || date.isEmpty) {
      return null
    }
    try {
      date.filter(_ != null).head

    } catch {
      case e: Exception => {
        null
      }
    }
  }

  /**
   * 获取最小的一个日期，如果有异常情况则反回第一个日期
   *
   * @param date
   * @return
   */
  def getMinDate(date: String*): String = {
    val notNullDate = date.filter(_ != null)
    if (notNullDate.map(_.length).distinct.length != 1) {
      return notNullDate.head
    }
    var minDate: String = null
    for (i <- 0 until notNullDate.length - 1) {
      minDate = getMaxDateBy2(notNullDate(i), notNullDate(i + 1), true)
    }
    minDate
  }

  /**
   * 获取最大的一个日期，如果有异常情况则反回第一个日期
   *
   * @param date
   */
  def getMaxDate(date: String*): String = {
    val notNullDate = date.filter(_ != null)
    if (notNullDate.map(_.length).distinct.length != 1) {
      return notNullDate.head
    }
    var maxDate: String = null
    for (i <- 0 until notNullDate.length - 1) {
      maxDate = getMaxDateBy2(notNullDate(i), notNullDate(i + 1))
    }
    maxDate
  }

  private def getMaxDateBy2(date1: String, date2: String, reverse: Boolean = false): String = {
    try {
      if (date1.length != date2.length) {
        return date1
      }
      val re = if (reverse) -1 else 1
      date1.compareTo(date2) * re match {
        case -1 => date2
        case 1 => date1
        case _ => date1
      }
    } catch {
      case e: Exception => date1
    }
  }

  def isLegalDate(date: String): Boolean = {
    try {
      var p = "yyyy-MM-dd HH:mm:ss"
      if (date.length == 10) {
        p = "yyyy-MM-dd"
      }
      val fm = new SimpleDateFormat(p)
      val date1 = fm.parse(date)
      date.equals(fm.format(date1))
    } catch {
      case e: Exception => false
    }
  }

  /**
   * 判断bizdate是否合法,否则用update_time替代
   *
   * @param date1
   * @param date2
   * @return
   */
  def getBizDate(date1: String, date2: String): String = {
    if (StringUtils.isBlank(date1)) {
      return date2
    }
    val s = date1.replaceAll("年", "-").replaceAll("月", "-")
      .replaceAll("日", "").replaceAll("/", "-")

    if (s.length >= 10 && isLegalDate(s.substring(0, 10))) {
      s.substring(0, 10)
    } else {
      date2
    }
  }

  def main(args: Array[String]): Unit = {
    println(DateUtils.toMillisTimestamp(date = "2001-06-05 00:00:00"))
    //    println(getNotNullStr(null, null))
    //    println(getNotNullStr(null, "2003-10-12 10:00:00", null, "2003-11-12 00:00:02"))
  }

}
